@@ -1,6 +1,6 @@
 from . import onnx  # noqa: F401
 from .__about__ import __copyright__, __license__, __version__  # noqa: F401
-from .closure import Add, Constant, Lambda, Multiply, One, Unity, Zero  # noqa: F401
+from .closure import Add, Constant, Identity, Lambda, Multiply, One, Zero  # noqa: F401
 from .container import (  # noqa: F401
     Broadcast,
     BroadcastReduce,
@@ -36,6 +36,7 @@ from .positional_encoding import RecyclingPositionalEncoding  # noqa: F401
 from .ptflops import _register_ptflops  # noqa: F401
 from .rnn import GRU, LSTM, RNN  # noqa: F401
 from .shape import Reshape  # noqa: F401
+from .skip import Skip  # noqa: F401
 from .transformer import (  # noqa: F401
     RetroactiveTransformerEncoderLayer,
     SingleOutputTransformerEncoderLayer,
