@@ -10,6 +10,18 @@ from torch import Tensor, nn
 
 from .logging import getLogger
 
+__all__ = [
+    "rsetattr",
+    "rgetattr",
+    "temporary_parameter",
+    "flat_state_dict",
+    "state_dict",
+    "load_state_dict",
+    "flatten",
+    "num_from",
+    "function_repr",
+]
+
 logger = getLogger(__name__)
 
 
@@ -233,3 +245,7 @@ def _flatten(lst: Union[Sequence[Any], Any]) -> List[Any]:
         return _flatten(first) + _flatten(rest)
     else:
         return [lst]
+
+
+def neginf_like(x: Tensor):
+    return -torch.ones_like(x) * float("inf")
